@@ -41,13 +41,13 @@ from .conda_interface import Resolve, MatchSpec, Unsatisfiable
 from .conda_interface import TemporaryDirectory
 from .conda_interface import get_rc_urls, get_local_urls
 from .conda_interface import VersionOrder
-from .conda_interface import (PaddingError, LinkError, CondaValueError, CondaError,
-                              NoPackagesFoundError, NoPackagesFound, LockError)
+from .conda_interface import (PaddingError, LinkError, CondaError, NoPackagesFoundError,
+                              NoPackagesFound, LockError)
 from .conda_interface import text_type
 from .conda_interface import CrossPlatformStLink
 from .conda_interface import PathType, FileMode
 from .conda_interface import EntityEncoder
-from .conda_interface import dist_str_in_index
+from .conda_interface import dist_str_in_index, Dist
 
 from conda_build import __version__
 from conda_build import environ, source, tarcheck, utils
@@ -353,7 +353,7 @@ def write_info_files_file(m, files, config):
                 fo.write(f + '\n')
 
 
-def write_package_metadata_json(m, config):
+def write_link_json(m, config):
     package_metadata = OrderedDict()
 
     noarch_type = m.get_value('build/noarch')
@@ -373,8 +373,11 @@ def write_package_metadata_json(m, config):
             preferred_env_dict["executable_paths"] = executable_paths
         package_metadata["preferred_env"] = preferred_env_dict
     if package_metadata:
+        # The original name of this file was info/package_metadata_version.json, but we've
+        #   now changed it to info/link.json.  Still, we must indefinitely keep the key name
+        #   package_metadata_version, or we break conda.
         package_metadata["package_metadata_version"] = 1
-        with open(os.path.join(config.info_dir, "package_metadata.json"), 'w') as fh:
+        with open(os.path.join(config.info_dir, "link.json"), 'w') as fh:
             fh.write(json.dumps(package_metadata, sort_keys=True, indent=2, separators=(',', ': ')))
 
 
@@ -433,7 +436,10 @@ def write_info_json(m, config):
 # It can be used to create the runtime environment of this package using:
 # $ conda create --name <env> --file <this file>
 """ % (m.dist(), config.subdir))
-            for dist in sorted(dists + [m.dist()]):
+            dist = m.dist()
+            if hasattr(dists[0], 'version'):
+                dist = Dist(dist)
+            for dist in sorted(dists + [dist]):
                 fo.write('%s\n' % '='.join(dist.split('::', 1)[-1].rsplit('-', 2)))
         if pin_depends == 'strict':
             info_index['depends'] = [' '.join(dist.split('::', 1)[-1].rsplit('-', 2))
@@ -488,7 +494,7 @@ def create_info_files(m, files, config, prefix):
 
     write_info_json(m, config)  # actually index.json
     write_about_json(m, config)
-    write_package_metadata_json(m, config)
+    write_link_json(m, config)
 
     write_info_files_file(m, files, config)
 
@@ -1207,6 +1213,7 @@ def warn_on_use_of_SRC_DIR(metadata):
                              " documentation regarding the test/source_files meta.yaml section, "
                              "or pass the --no-remove-work-dir flag.")
 
+
 def test(recipedir_or_package_or_metadata, config, move_broken=True):
     '''
     Execute any test scripts for the given package.
@@ -1427,9 +1434,9 @@ def build_tree(recipe_list, config, build_only=False, post=False, notest=False,
             subprocess.call('del /s /q "{0}\\*.*" >nul 2>&1'.format(trash_dir), shell=True)
         # delete_trash(None)
 
-    already_built = set()
     extra_help = ""
     built_packages = []
+    retried_recipes = []
 
     while recipe_list:
         # This loop recursively builds dependencies if recipes exist
@@ -1459,9 +1466,6 @@ def build_tree(recipe_list, config, build_only=False, post=False, notest=False,
             recipe = recipe.rstrip("/").rstrip("\\")
             to_build_recursive.append(os.path.basename(recipe))
 
-            #    before downloading happens - or else we lose where downloads are
-            if config.set_build_id:
-                config.compute_build_id(os.path.basename(recipe), reset=True)
             metadata, need_source_download, need_reparse_in_env = render_recipe(recipe,
                                                                     config=config)
         if not getattr(config, "noverify", False):
@@ -1501,7 +1505,7 @@ def build_tree(recipe_list, config, build_only=False, post=False, notest=False,
                 pkg = pkg.strip().split(' ')[0]
 
                 if pkg in to_build_recursive:
-                    raise RuntimeError("Can't build {0} due to unsatisfiable dependencies:\n"
+                    raise RuntimeError("Can't build {0} due to environment creation error:\n"
                                        .format(recipe) + error_str + "\n" + extra_help)
 
                 if pkg in skip_names:
@@ -1521,46 +1525,52 @@ packages, the other package needs to be rebuilt
                 else:
                     raise RuntimeError("Can't build {0} due to unsatisfiable dependencies:\n{1}"
                                        .format(recipe, e.pkgs) + "\n\n" + extra_help)
+            if retried_recipes.count(recipe) >= len(metadata.ms_depends('build')):
+                raise RuntimeError("Can't build {0} due to environment creation error:\n"
+                                    .format(recipe) + error_str + "\n" + extra_help)
+            retried_recipes.append(recipe)
             recipe_list.extendleft(add_recipes)
 
-        # outputs message, or does upload, depending on value of args.anaconda_upload
-        if post in [True, None]:
-            for f in built_packages:
-                # TODO: could probably use a better check for pkg type than this...
-                if f.endswith('.tar.bz2'):
-                    handle_anaconda_upload(f, config=config)
-                elif f.endswith('.whl'):
-                    handle_pypi_upload(f, config=config)
-                already_built.add(f)
-
+    if post in [True, None]:
+        # TODO: could probably use a better check for pkg type than this...
+        tarballs = [f for f in built_packages if f.endswith('.tar.bz2')]
+        wheels = [f for f in built_packages if f.endswith('.whl')]
+        handle_anaconda_upload(tarballs, config=config)
+        handle_pypi_upload(wheels, config=config)
     return built_packages
 
 
-def handle_anaconda_upload(path, config):
+def handle_anaconda_upload(paths, config):
     from conda_build.os_utils.external import find_executable
 
+    paths = utils.ensure_list(paths)
+
     upload = False
     # this is the default, for no explicit argument.
     # remember that anaconda_upload takes defaults from condarc
     if config.anaconda_upload is None:
         pass
+    elif config.token or config.user:
+        upload = True
     # rc file has uploading explicitly turned off
     elif config.anaconda_upload is False:
         print("# Automatic uploading is disabled")
+        upload = False
     else:
         upload = True
 
-    if config.token or config.user:
-        upload = True
-
     no_upload_message = """\
-# If you want to upload this package to anaconda.org later, type:
-#
-# $ anaconda upload %s
-#
+# If you want to upload package(s) to anaconda.org later, type:
+
+"""
+    for package in paths:
+        no_upload_message += "anaconda upload {}\n".format(package)
+
+    no_upload_message += """\
+
 # To have conda build upload to anaconda.org automatically, use
 # $ conda config --set anaconda_upload yes
-""" % path
+"""
     if not upload:
         print(no_upload_message)
         return
@@ -1573,7 +1583,6 @@ Error: cannot locate anaconda command (required for upload)
 # Try:
 # $ conda install anaconda-client
 ''')
-    print("Uploading to anaconda.org")
     cmd = [anaconda, ]
 
     if config.token:
@@ -1581,15 +1590,16 @@ Error: cannot locate anaconda command (required for upload)
     cmd.extend(['upload', '--force'])
     if config.user:
         cmd.extend(['--user', config.user])
-    cmd.append(path)
-    try:
-        subprocess.call(cmd)
-    except:
-        print(no_upload_message)
-        raise
+    for package in paths:
+        try:
+            print("Uploading {} to anaconda.org".format(os.path.basename(package)))
+            subprocess.call(cmd + [package])
+        except subprocess.CalledProcessError:
+            print(no_upload_message)
+            raise
 
 
-def handle_pypi_upload(f, config):
+def handle_pypi_upload(wheels, config):
     args = ['twine', 'upload', '--sign-with', config.sign_with, '--repository', config.repository]
     if config.user:
         args.extend(['--user', config.user])
@@ -1604,12 +1614,14 @@ def handle_pypi_upload(f, config):
     if config.repository:
         args.extend(['--repository', config.repository])
 
-    args.append(f)
-    try:
-        utils.check_call_env(args)
-    except:
-        logging.getLogger(__name__).warn("wheel upload failed - is twine installed?"
-                                         "  Is this package registered?")
+    wheels = utils.ensure_list(wheels)
+
+    for f in wheels:
+        try:
+            utils.check_call_env(args + [f])
+        except:
+            logging.getLogger(__name__).warn("wheel upload failed - is twine installed?"
+                                            "  Is this package registered?")
 
 
 def print_build_intermediate_warning(config):
