@@ -449,7 +449,7 @@ class MetaData(object):
         """For dynamic determination of build or run reqs, based on configuration"""
         reqs = self.meta.get('requirements', {})
         run_reqs = reqs.get('run', [])
-        build_reqs = reqs.get('build', [])
+        # build_reqs = reqs.get('build', [])
         if bool(self.get_value('build/osx_is_app', False)) and self.config.platform == 'osx':
             run_reqs.append('python.app')
         self.meta['requirements'] = reqs
