@@ -19,7 +19,7 @@ from conda_build.conda_interface import get_index
 from conda_build.conda_interface import TmpDownload, download
 from conda_build.conda_interface import MatchSpec, Resolve
 from conda_build.conda_interface import memoized
-from conda_build.conda_interface import CondaHTTPError
+from conda_build.conda_interface import CondaHTTPError, CondaError
 
 from conda_build.config import Config
 from conda_build.utils import on_win, check_call_env
@@ -76,10 +76,10 @@ CPAN_BUILD_SH = """\
 # If it has Build.PL use that, otherwise use Makefile.PL
 if [ -f Build.PL ]; then
     perl Build.PL
-    ./Build
-    ./Build test
+    perl ./Build
+    perl ./Build test
     # Make sure this goes in site
-    ./Build install --installdirs site
+    perl ./Build install --installdirs site
 elif [ -f Makefile.PL ]; then
     # Make sure this goes in site
     perl Makefile.PL INSTALLDIRS=site
@@ -166,6 +166,8 @@ def get_cpan_api_url(url, colons):
             rel_dict = json.loads(output)
         except IOError:
             rel_dict = json.loads(codecs.open(json_path, encoding='utf-8').read())
+        except CondaHTTPError:
+            rel_dict = None
     return rel_dict
 
 
@@ -182,9 +184,10 @@ def package_exists(package_name):
     return in_repo
 
 
+# meta_cpan_url="http://api.metacpan.org",
 def skeletonize(packages, output_dir=".", version=None,
-                meta_cpan_url="http://api.metacpan.org",
-                recursive=False, config=None):
+                meta_cpan_url="http://fastapi.metacpan.org/v1",
+                recursive=False, force=False, config=None):
     '''
     Loops over packages, outputting conda recipes converted from CPAN metata.
     '''
@@ -254,10 +257,40 @@ def skeletonize(packages, output_dir=".", version=None,
                                          core_version),
                                          perl_version,
                                          config=config)
+
         # Check if recipe directory already exists
-        dir_path = join(output_dir, packagename)
-        if exists(dir_path):
-            raise RuntimeError("directory already exists: %s" % dir_path)
+        dir_path = join(output_dir, packagename, release_data['version'])
+
+        # Add Perl version to core module requirements, since these are empty
+        # packages, unless we're newer than what's in core
+        if core_version is not None and ((version in [None, '']) or
+                                            (core_version >=
+                                            LooseVersion(version))):
+            d['useurl'] = '#'
+            d['usemd5'] = '#'
+            d['source_comment'] = '#'
+            empty_recipe = True
+        # Add dependencies to d if not in core, or newer than what's in core
+        else:
+            build_deps, run_deps, packages_to_append = deps_for_package(
+                package, release_data=release_data, perl_version=perl_version,
+                output_dir=output_dir, meta_cpan_url=meta_cpan_url,
+                recursive=recursive, config=config)
+            d['build_depends'] += indent.join([''] + list(build_deps |
+                                                            run_deps))
+            d['run_depends'] += indent.join([''] + list(run_deps))
+            # Make sure we append any packages before continuing
+            packages.extend(packages_to_append)
+            empty_recipe = False
+
+        # If we are recursively getting packages for a particular version
+        # we need to make sure this is reset on the loop
+        version = None
+        if exists(dir_path) and not force:
+            print('Directory %s already exists and you have not specified --force ' % dir_path)
+            continue
+        elif exists(dir_path) and force:
+            print('Directory %s already exists, but forcing recipe creation' % dir_path)
 
         # If this is something we're downloading, get MD5
         if release_data['download_url']:
@@ -286,27 +319,6 @@ def skeletonize(packages, output_dir=".", version=None,
 
         processed_packages.add(packagename + '-' + d['version'])
 
-        # Add Perl version to core module requirements, since these are empty
-        # packages, unless we're newer than what's in core
-        if core_version is not None and ((version in [None, '']) or
-                                            (core_version >=
-                                            LooseVersion(version))):
-            d['useurl'] = '#'
-            d['usemd5'] = '#'
-            d['source_comment'] = '#'
-            empty_recipe = True
-        # Add dependencies to d if not in core, or newer than what's in core
-        else:
-            build_deps, run_deps, packages_to_append = deps_for_package(
-                package, release_data=release_data, perl_version=perl_version,
-                output_dir=output_dir, meta_cpan_url=meta_cpan_url,
-                recursive=recursive, config=config)
-            d['build_depends'] += indent.join([''] + list(build_deps |
-                                                            run_deps))
-            d['run_depends'] += indent.join([''] + list(run_deps))
-            packages.extend(packages_to_append)
-            empty_recipe = False
-
         # Create import tests
         module_prefix = package.replace('::', '-').split('-')[0]
         if 'provides' in release_data:
@@ -320,8 +332,10 @@ def skeletonize(packages, output_dir=".", version=None,
         else:
             d['import_comment'] = '# '
 
+        if not exists(dir_path):
+            makedirs(dir_path)
+
         # Write recipe files to a directory
-        makedirs(dir_path)
         print("Writing recipe for %s-%s" % (packagename, d['version']))
         with open(join(dir_path, 'meta.yaml'), 'w') as f:
             f.write(CPAN_META.format(**d))
@@ -357,12 +371,16 @@ def add_parser(repos):
         help="Version to use. Applies to all packages.",)
     cpan.add_argument(
         "--meta-cpan-url",
-        default='http://api.metacpan.org',
-        help="URL to use for MetaCPAN API.",)
+        default='http://fastapi.metacpan.org/v1',
+        help="URL to use for MetaCPAN API. It must include a version, such as v1",)
     cpan.add_argument(
         "--recursive",
         action='store_true',
         help='Create recipes for dependencies if they do not already exist.')
+    cpan.add_argument(
+        "--force",
+        action='store_true',
+        help='Force overwrite of existing recipes')
 
 
 @memoized
@@ -373,7 +391,7 @@ def latest_pkg_version(pkg):
     r = Resolve(get_index())
     try:
         pkg_list = sorted(r.get_pkgs(MatchSpec(pkg)))
-    except RuntimeError:
+    except:
         pkg_list = None
     if pkg_list:
         pkg_version = LooseVersion(pkg_list[-1].version)
@@ -525,7 +543,7 @@ def deps_for_package(package, release_data, perl_version, output_dir,
                                                                         '-'))):
                             packages_to_append.add('='.join((orig_dist,
                                                             dep_dict['version'])))
-                    elif not glob(join(output_dir, (dep_entry + '-[v0-9][0-9.]*'))):
+                    elif not glob(join(output_dir, (dep_entry + '-[v1-9][0-9.]*'))):
                         packages_to_append.add(orig_dist)
 
                 # Add to appropriate dependency list
@@ -537,7 +555,7 @@ def deps_for_package(package, release_data, perl_version, output_dir,
         # seemingly new in conda 4.3: HTTPErrors arise when we ask for something that is a
         #   perl module, but not a package.
         # See https://github.com/conda/conda-build/issues/1675
-        except CondaHTTPError:
+        except (CondaError, CondaHTTPError):
             continue
 
     return build_deps, run_deps, packages_to_append
@@ -551,19 +569,21 @@ def dist_for_module(cpan_url, module, perl_version, config):
     '''
     # First check if its already a distribution
     try:
-        rel_dict = get_cpan_api_url('{0}/v0/release/{1}'.format(cpan_url, module), colons=False)
+        rel_dict = get_cpan_api_url('{0}/release/{1}'.format(cpan_url, module), colons=False)
     # If there was an error, module may actually be a module
     except RuntimeError:
         rel_dict = None
+    except CondaHTTPError:
+        rel_dict = None
     else:
         distribution = module
 
     # Check if it's a module instead
     if rel_dict is None:
         try:
-            mod_dict = get_cpan_api_url('{0}/v0/module/{1}'.format(cpan_url, module), colons=True)
+            mod_dict = get_cpan_api_url('{0}/module/{1}'.format(cpan_url, module), colons=True)
         # If there was an error, report it
-        except RuntimeError as exc:
+        except CondaHTTPError as exc:
             core_version = core_module_version(module, perl_version, config=config)
             if core_version is None:
                 sys.exit(('Error: Could not find module or distribution named'
@@ -589,9 +609,9 @@ def get_release_info(cpan_url, package, version, perl_version, config,
     # Get latest info to find author, which is necessary for retrieving a
     # specific version
     try:
-        rel_dict = get_cpan_api_url('{0}/v0/release/{1}'.format(cpan_url, package), colons=False)
+        rel_dict = get_cpan_api_url('{0}/release/{1}'.format(cpan_url, package), colons=False)
         rel_dict['version'] = str(rel_dict['version']).lstrip('v')
-    except RuntimeError:
+    except CondaHTTPError:
         core_version = core_module_version(orig_package, perl_version, config=config)
         if core_version is not None and (version is None or
                                          (version == core_version)):
@@ -612,11 +632,11 @@ def get_release_info(cpan_url, package, version, perl_version, config,
             (LooseVersion(rel_dict['version']) != LooseVersion(version_str))):
         author = rel_dict['author']
         try:
-            new_rel_dict = get_cpan_api_url('{0}/v0/release/{1}/{2}-{3}'.format(cpan_url,
+            new_rel_dict = get_cpan_api_url('{0}/release/{1}/{2}-{3}'.format(cpan_url,
                                                 author, package, version_str), colons=False)
             new_rel_dict['version'] = new_rel_dict['version'].lstrip()
         # Check if this is a core module, and don't die if it is
-        except RuntimeError:
+        except CondaHTTPError:
             core_version = core_module_version(orig_package, perl_version, config=config)
             if core_version is not None and (version == core_version):
                 print(("WARNING: Version {0} of {1} is not available on " +
