@@ -418,7 +418,8 @@ def skeletonize(packages, output_dir=".", version=None, recursive=False,
 
         get_package_metadata(package, d, data, output_dir, python_version,
                              all_extras, recursive, created_recipes, noarch_python,
-                             noprompt, packages, extra_specs, config=config, setup_options=setup_options)
+                             noprompt, packages, extra_specs, config=config,
+                             setup_options=setup_options)
 
         if d['import_tests'] == '':
             d['import_comment'] = '# '
@@ -918,7 +919,8 @@ def get_requirements(package, pkginfo, all_extras=True):
     return requires
 
 
-def get_pkginfo(package, filename, pypiurl, md5, python_version, extra_specs, config, setup_options):
+def get_pkginfo(package, filename, pypiurl, md5, python_version, extra_specs, config,
+                setup_options):
     # Unfortunately, two important pieces of metadata are only stored in
     # the package itself: the dependencies, and the entry points (if the
     # package uses distribute).  Our strategy is to download the package
@@ -945,7 +947,8 @@ def get_pkginfo(package, filename, pypiurl, md5, python_version, extra_specs, co
         print("working in %s" % tempdir)
         src_dir = get_dir(tempdir)
         # TODO: find args parameters needed by run_setuppy
-        run_setuppy(src_dir, tempdir, python_version, extra_specs=extra_specs, config=config, setup_options=setup_options)
+        run_setuppy(src_dir, tempdir, python_version, extra_specs=extra_specs, config=config,
+                    setup_options=setup_options)
         try:
             with open(join(tempdir, 'pkginfo.yaml')) as fn:
                 pkg_info = yaml.load(fn)
